@@ -8,13 +8,13 @@ from math import ceil
 from .molecule import packmol
 from .dft.siesta import siesta_md 
 from .dft.SinglePointEnergy import SinglePointEnergies
-from .md.gmd import nvt_wt as gulp_nvt
+from .md.gulp import nvt as gulp_nvt
 from .data.mdtodata import MDtoData
 from .data.prep_data import prep_data
 from .reaxfflib import read_lib
 import tensorflow as tf
 from .md.irmd import IRMD
-from .training import train
+from .training import train_reax
 from .training import train_mpnn
 from .AtomDance import AtomDance,check_zmat
 from .dingtalk import send_msg
@@ -63,8 +63,8 @@ class LearningMachine(object):
                T=300,Tmax=1000,
                label=None,
                convergence=0.01,nconvergence=3,
-               accCriteria=0.95,lossCriteria=1000.0,lossTole=1.0,accTole=0.005,
-               accMax=0.96,accInc=1.0001,mdInc=1.2,resetAcc=1,
+               accCriteria=0.9,lossCriteria=1000.0,lossTole=1.0,accTole=0.005,
+               accMax=0.9,accInc=1.0001,mdInc=1.2,resetAcc=1,
                learnWay=5,FreePairs=None,beta=None,
                FreeAtoms=None,FirstAtom=None,
                rodic=None,
@@ -72,82 +72,95 @@ class LearningMachine(object):
                CheckZmat=True,
                EngTole=0.05,dEtole=0.2,dEstop=2.0,
                nn=True,vdwnn=False,
-               bo_layer=[2,1],mf_layer=[6,1],be_layer=[6,1],
-               EnergyFunction=1,MessageFunction=3,
-               bom={'others':1.0},
-               bore={'others':0.45},
+               bo_layer=[2,1],mf_layer=[9,2],be_layer=[6,1],vdw_layer=None,#[6,1],
+               be_univeral_nn='all',bo_univeral_nn='all',mf_univeral_nn='all',vdw_univeral_nn='all',
+               EnergyFunction=1,MessageFunction=1,
+               spv_bm=False,bom={'others':1.0},
+               spv_be=True,bore={'others':0.2},
+               spv_ang=True,lambda_ang=0.02,
                weight={'others':2.0},
-               spv_bm=False,spv_be=False,
-               lambda_bd=10000.0,lambda_reg=0.000001,regularize=True,
-               mefac=0.1,
-               writelib=5000,
+               lambda_bd=10000.0,lambda_reg=0.0001,regularize=True,
+               lambda_me=0.1,learning_rate=0.0001,
+               writelib=5000,ffield='ffield.json',
                xcf='VDW',xca='DRSLL',basistype='DZP'):
       ''' max_batch: max number of batch 
               batch: batch size
       '''
-      self.initConfig   = initConfig
-      self.label        = initConfig.split('.')[0]
-      self.aidir        = 'aimd_' + self.label
-      self.accCriteria  = accCriteria # accuracy convergence criteria
-      self.lossCriteria = lossCriteria    
-      self.lossTole     = lossTole    
-      self.accTole      = accTole      # Minimum accuracy convergence after a training process
-      self.accMax       = accMax 
-      self.accInc       = accInc      # accuracy inrease factor
-      self.resetAcc     = resetAcc    # reset accuracy criteria after this train cycle
-      self.rmin         = rmin       # minimum bond length used to supervise ML-MD simulation
-      self.rmax         = rmax
-      self.angmax       = angmax
-      self.CheckZmat    = CheckZmat
-      self.convergence  = convergence
-      self.nconvergence = nconvergence
-      self.learnWay     = learnWay    # the way of learing
-      self.ncpu         = ncpu
-      self.step         = step
-      self.batch        = batch
-      self.direcs       = direcs
-      self.maxiter      = maxiter
-      self.max_batch    = max_batch   # max number in direcs to train
-      self.xcf          = xcf
-      self.xca          = xca
-      self.basistype    = basistype
-      self.dft_step     = dft_step
-      self.dt_mlmd      = dt_mlmd
-      self.dt_aimd      = dt_aimd
-      self.T            = T
-      self.Tmax         = Tmax
-      self.md_step      = md_step
-      self.mom_step     = mom_step
-      self.beta         = beta
-      self.colFrame     = colFrame
-      self.EngTole      = EngTole
-      self.dEtole       = dEtole
-      self.dEstop       = dEstop
-      self.MaxMDstep    = MaxMDstep
-      self.mpopt        = mpopt
-      self.optword      = optword
-      self.messages     = messages
-      self.mdInc        = mdInc       # MD step increase factor
-      self.nn           = nn
-      self.vdwnn        = vdwnn
-      self.bo_layer     = bo_layer
-      self.mf_layer     = mf_layer
-      self.be_layer     = be_layer
-      self.bore         = bore
-      self.bom          = bom
-      self.spv_bm       = spv_bm
-      self.spv_be       = spv_be
-      self.mefac        = mefac
-      self.weight       = weight
-      self.lambda_bd    = lambda_bd
-      self.regularize   = regularize
-      self.lambda_reg   = lambda_reg
+      self.initConfig     = initConfig
+      self.label          = initConfig.split('.')[0]
+      self.aidir          = 'aimd_' + self.label
+      self.accCriteria    = accCriteria # accuracy convergence criteria
+      self.lossCriteria   = lossCriteria    
+      self.lossTole       = lossTole    
+      self.accTole        = accTole      # Minimum accuracy convergence after a training process
+      self.accMax         = accMax 
+      self.accInc         = accInc      # accuracy inrease factor
+      self.resetAcc       = resetAcc    # reset accuracy criteria after this train cycle
+      self.rmin           = rmin       # minimum bond length used to supervise ML-MD simulation
+      self.rmax           = rmax
+      self.angmax         = angmax
+      self.CheckZmat      = CheckZmat
+      self.convergence    = convergence
+      self.nconvergence   = nconvergence
+      self.learnWay       = learnWay    # the way of learing
+      self.ncpu           = ncpu
+      self.step           = step
+      self.batch          = batch
+      self.direcs         = direcs
+      self.maxiter        = maxiter
+      self.max_batch      = max_batch   # max number in direcs to train
+      self.xcf            = xcf
+      self.xca            = xca
+      self.basistype      = basistype
+      self.dft_step       = dft_step
+      self.dt_mlmd        = dt_mlmd
+      self.dt_aimd        = dt_aimd
+      self.T              = T
+      self.Tmax           = Tmax
+      self.md_step        = md_step
+      self.mom_step       = mom_step
+      self.beta           = beta
+      self.colFrame       = colFrame
+      self.EngTole        = EngTole
+      self.dEtole         = dEtole
+      self.dEstop         = dEstop
+      self.MaxMDstep      = MaxMDstep
+      self.mpopt          = mpopt
+      self.optword        = optword
+      self.messages       = messages
+      self.mdInc          = mdInc       # MD step increase factor
+      self.ffield         = ffield
+      self.nn             = nn
+      self.vdwnn          = vdwnn
+      if not self.nn:
+         self.vdwnn       = False
+      self.bo_layer       = bo_layer
+      self.mf_layer       = mf_layer
+      self.be_layer       = be_layer
+      self.vdw_layer      = vdw_layer
+      self.bo_univeral_nn = bo_univeral_nn
+      self.be_univeral_nn = be_univeral_nn
+      self.mf_univeral_nn = mf_univeral_nn
+      self.vdw_univeral_nn= vdw_univeral_nn
+      self.bore           = bore
+      self.bom            = bom
+      self.spv_bm         = spv_bm
+      self.spv_be         = spv_be
+      self.spv_ang        = spv_ang
+      self.lambda_me      = lambda_me
+      self.weight         = weight
+      self.lambda_bd      = lambda_bd
+      self.regularize     = regularize
+      self.lambda_reg     = lambda_reg
+      self.lambda_ang     = lambda_ang
+      self.learning_rate  = learning_rate
       self.EnergyFunction = EnergyFunction
       self.MessageFunction= MessageFunction
-      self.writelib     = writelib
-      self.freeatoms    = FreeAtoms
-      self.freepairs    = FreePairs
-      self.a            = AtomDance(poscar=self.initConfig,nn=self.nn,
+      self.writelib       = writelib
+      self.freeatoms      = FreeAtoms
+      self.freepairs      = FreePairs
+      self.a              = AtomDance(poscar=self.initConfig,nn=self.nn,
+                                    ffield=ffield,
                                     rmin=self.rmin-0.05,rmax=self.rmax,
                                     FirstAtom=FirstAtom,freeatoms=self.freeatoms)
       if self.freepairs is None:
@@ -169,7 +182,7 @@ class LearningMachine(object):
       if self.nn:
          self.trainer = train_mpnn
       else:
-         self.trainer = train
+         self.trainer = train_reax
       
       if label is None:
          aimd_ = self.aidir.split('_')
@@ -318,6 +331,7 @@ class LearningMachine(object):
           while (accu<=self.accCriteria or loss>=self.lossCriteria) and training:        
                 self.load_config()
                 loss,accu,accMax,p,zpe,tstep = self.trainer(direcs=trajs_,
+                                                     ffield=self.ffield,
                                                      step=self.step,writelib=self.writelib,
                                                      batch=self.batch,
                                                      convergence=self.accCriteria,
@@ -327,13 +341,20 @@ class LearningMachine(object):
                                                      bo_layer=self.bo_layer,
                                                      mf_layer=self.mf_layer,
                                                      be_layer=self.be_layer,
+                                                     vdw_layer=self.vdw_layer,
+                                                     bo_univeral_nn=self.bo_univeral_nn,
+                                                     be_univeral_nn=self.be_univeral_nn,
+                                                     mf_univeral_nn=self.mf_univeral_nn,
+                                                     vdw_univeral_nn=self.vdw_univeral_nn,
                                                      EnergyFunction=self.EnergyFunction,
                                                      MessageFunction=self.MessageFunction,
                                                      bore=self.bore, bom=self.bom,
                                                      spv_be=self.spv_be,spv_bm=self.spv_bm,
-                                                     mefac=self.mefac,weight=self.weight,
+                                                     spv_ang=self.spv_ang,lambda_ang=self.lambda_ang,
+                                                     lambda_me=self.lambda_me,weight=self.weight,
                                                      regularize=self.regularize,lambda_reg=self.lambda_reg,
-                                                     lambda_bd=self.lambda_bd)
+                                                     lambda_bd=self.lambda_bd,
+                                                     learning_rate=self.learning_rate)
                 self.load_config()
                 if tIter>=self.resetAcc and tstep>=self.step:
                    if accu>self.accCriteria-self.accTole and loss<self.lossCriteria+self.lossTole:
@@ -371,9 +392,9 @@ class LearningMachine(object):
                 e_gmd,mdsteps,Deformed,zmats,zmat_variable,zvlo,zvhi = self.mlmd(cwd,run_dir,atoms, 10000,
                                                 iter_,learnWay,beta=self.beta,learnpair=self.learnpair,
                                                 groupi=groupi,groupj=groupj) 
-                # if self.CheckZmat and mdsteps<=2:
-                #    mdsteps= 7
-                #    images,relaxlog = self.a.zmat_relax(atoms=atoms,nbin=20,relax_step=mdsteps,traj='md.traj')
+                if self.CheckZmat and mdsteps<=3:
+                   mdsteps= 10
+                   images,relaxlog = self.a.continous(atoms,nbin=mdsteps,traj='md.traj')
              if not images is None:
                 Deformed,zmat,zmat_variable,zvlo,zvhi = check_zmat(atoms=images[-1],rmin=self.rmin,
                                        rmax=self.rmax,angmax=self.angmax,zmat_id=self.a.zmat_id,
@@ -473,8 +494,9 @@ class LearningMachine(object):
       mkdir(run_dir)
       chdir(cwd+'/'+run_dir)
       popen('cp ../../*.psf ./')
-      emlmd = []
-      dEmax = 0.0
+      emlmd  = []
+      ind_   = []
+      dEmax  = 0.0
       d2Emax = 0.0
 
       if learnWay==1:
@@ -560,10 +582,9 @@ class LearningMachine(object):
                     'angmax':self.angmax,
                     'accTole':self.accTole,
                     'accMax':self.accMax,
-                    'accInc':self.accInc,
-                    'resetAcc':self.resetAcc,
+                    # 'accInc':self.accInc,
+                    # 'resetAcc':self.resetAcc,
                     'step':self.step,
-                    'dft_step':self.dft_step,
                     'md_step':self.md_step,
                     'mom_step':self.mom_step,
                     'MaxMDstep':self.MaxMDstep,
@@ -571,17 +592,18 @@ class LearningMachine(object):
                     'dEtole':self.dEtole,
                     'dEstop':self.dEstop,
                     'beta':self.beta,
-                    'mdInc':self.mdInc,
+                    # 'mdInc':self.mdInc,
                     'regularize':self.regularize,
                     'lambda_reg':self.lambda_reg,
                     'lambda_bd':self.lambda_bd,
+                    'lambda_me':self.lambda_me,
+                    'lambda_ang':self.lambda_ang,
+                    'learning_rate':self.learning_rate,
                     'rmin':self.rmin,
-                    'Tmax':self.Tmax,
                     'T':self.T,
                     'maxiter':self.maxiter,
                     'max_batch':self.max_batch,
                     'optword':self.optword,
-                    'mefac':self.mefac,
                     'learnWay':self.learnWay,
                     'colFrame':self.colFrame,
                     'CheckZmat':self.CheckZmat,
@@ -589,6 +611,8 @@ class LearningMachine(object):
                     'freepairs':self.freepairs,
                     'bore':self.bore,
                     'writelib':self.writelib}
+           if self.learnWay==1:
+              options['dft_step'] = self.dft_step
            js.dump(options,fj,sort_keys=True,indent=2) 
 
   def load_config(self):
@@ -600,11 +624,12 @@ class LearningMachine(object):
            self.rmax         = options['rmax']
            self.angmax       = options['angmax']
            self.accTole      = options['accTole']
-           self.accMax       = options['accMax']
-           self.accInc       = options['accInc']
-           self.resetAcc     = options['resetAcc']
+           self.accMax     = options['accMax']
+           # self.accInc     = options['accInc']
+           # self.resetAcc   = options['resetAcc']
            self.step         = options['step'] 
-           self.dft_step     = options['dft_step'] 
+           if self.learnWay==1:
+              self.dft_step     = options['dft_step'] 
            self.md_step      = options['md_step'] 
            self.mom_step     = options['mom_step'] 
            self.MaxMDstep    = options['MaxMDstep'] 
@@ -612,18 +637,19 @@ class LearningMachine(object):
            self.dEtole       = options['dEtole'] 
            self.dEstop       = options['dEstop'] 
            self.beta         = options['beta'] 
-           self.mdInc        = options['mdInc'] 
+           # self.mdInc      = options['mdInc'] 
            self.regularize   = options['regularize'] 
            self.lambda_reg   = options['lambda_reg'] 
            self.lambda_bd    = options['lambda_bd']
+           self.lambda_me    = options['lambda_me']
+           self.lambda_ang   = options['lambda_ang']
            self.rmin         = options['rmin'] 
-           self.Tmax         = options['Tmax'] 
            self.T            = options['T'] 
            self.maxiter      = options['maxiter']
            self.max_batch    = options['max_batch']
            self.optword      = options['optword']
-           self.mefac        = options['mefac']
            self.learnWay     = options['learnWay']
+           self.learning_rate= options['learning_rate']
            self.colFrame     = options['colFrame']
            self.CheckZmat    = options['CheckZmat']
            self.bore         = options['bore']
