@@ -93,8 +93,8 @@ class Init_Check(object):
           if k == 'boc3':
              if p[key]<=0.001:
                 p[key]= value(0.001,key)
-             if p[key]>=40.0:
-                p[key]= value(20.0,key)  
+             if p[key]>=50.0:
+                p[key]= value(50.0,key)  
 
           if key == 'lp1':
              if p[key]>=28.0:
@@ -118,8 +118,8 @@ class Init_Check(object):
                 p[key] = value(10.0,key)
 
           if k=='val1': 
-             if p[key]>=90.0:
-                p[key] = value(0.0,key)
+             if p[key]>=200.0:
+                p[key] = value(200.0,key)
              pr = key.split('_')[1]
              ang= pr.split('-')
              if  ang[1]=='H':
@@ -129,37 +129,41 @@ class Init_Check(object):
                    p[key] = value(1.0,key)
 
           if k=='val2': 
-             if p[key]>=30.0:
-                p[key] = value(15.0,key)
+             if p[key]>=8.0:
+                p[key] = value(1.0,key)
              if p[key]<=0.001:
                 p[key] = value(0.01,key)
           if k=='val3': 
-             if p[key]<=0.001:
-                p[key] = value(1.0,key)
+             # if p[key]<=0.001:
+             #    p[key] = value(1.0,key)
              if p[key]>=12.0:
                 p[key] = value(10.0,key)
 
           if k=='val4': 
-             if p[key]<=0.001:
-                p[key] = value(1.0,key)
+             # if p[key]<=0.001:
+             #    p[key] = value(1.0,key)
              if p[key]>=10.0:
                 p[key] = value(5.0,key)
           if k=='val7': 
+             pr = key.split('_')[1]
+             ang= pr.split('-')
+             if  ang[1]=='H':
+                 p[key] = value(0.0,key)
              if p[key]>=30.0:
                 p[key] = value(15.0,key)
           if k=='val8': 
-             if p[key]>=3.0:
-                p[key] = value(1.0,key)
+             if p[key]>=6.0:
+                p[key] = value(6.0,key)
              if p[key]<=0.1:
                 p[key] = value(1.0,key)
           if k=='val9': 
-             if p[key]>=3.0:
-                p[key] = value(1.0,key)
-             if p[key]<=0.1:
-                p[key] = value(1.0,key)
+             if p[key]>=8.0:
+                p[key] = value(8.0,key)
+             # if p[key]<=0.1:
+             #    p[key] = value(1.0,key)
           if k=='val10': 
-             if p[key]>=5.0:
-                p[key] = value(3.0,key)
+             if p[key]>=15.0:
+                p[key] = value(15.0,key)
              if p[key]<=0.1:
                 p[key] = value(1.0,key)  
 
@@ -203,20 +207,20 @@ class Init_Check(object):
                 p[key] = value(3.0,key)
 
           if k=='alfa': 
-             if p[key]>=18.0:
+             if p[key]>=20.0:
                 p[key] = value(12.0,key)
              if p[key]<=1.0:
                 p[key] = value(3.0,key)
 
           if k=='gammaw': 
              if p[key]>=10.0:
-                p[key] = value(6.0,key)
-             if p[key]<=0.8:
-                p[key] = value(1.5,key)
+                p[key] = value(10.0,key)
+             if p[key]<=0.5:
+                p[key] = value(0.5,key)
 
           if k=='Desi': 
-             if p[key]<60.0:
-                p[key]= value(70.0,key)
+             if p[key]<10.0:
+                p[key]= value(10.0,key)
              if p[key]>=800.0:
                 p[key]= value(600.0,key)
           if k=='Depi': 
@@ -230,8 +234,8 @@ class Init_Check(object):
              if p[key]>=600.0:
                 p[key]= value(600.0,key)
           if k=='Devdw': 
-             if p[key]>=8.0:
-                p[key]= value(8.0,key)
+             # if p[key]>=20.0:
+             #    p[key]= value(8.0,key)
              if p[key]<=0.01:
                 p[key]= value(0.01,key)
           if k=='vdw1': 
@@ -246,13 +250,15 @@ class Init_Check(object):
                 p[key]= value(2.0,key)
           if k=='hb1': 
              if p[key]>=25.0:
-                p[key]= value(2.0,key)
+                p[key]= value(25.0,key)
           if k=='hb2': 
              if p[key]>=25.0:
-                p[key]= value(20.0,key)
+                p[key]= value(25.0,key)
           if k=='ovun8': 
              if p[key]>=15.0:
                 p[key] = value(10.0,key)
+             if p[key]<=-15.0:
+                p[key] = value(-10.0,key)
           if k=='val7': 
              if p[key]>=20.0:
                 p[key] = value(18.0,key)
@@ -269,22 +275,22 @@ class Init_Check(object):
                 p[key]= value(1.0,key)
              if p[key]>=20.00:
                 p[key]= value(2.0,key)
-          if k=='ovun5': 
-             if p[key]>=60.0:
-                p[key]= value(59.0,key)
-          if k in ['bo2','bo4','bo6']:     ### scale bo
-             if p[key]>=8.0:
-                p[key] = value(7.95,key)
-             if p[key]<=2.0:
-                p[key] = value(2.05,key)
-          if k in ['bo1','bo3','bo5']:     ### scale bo
-             if p[key]<=-0.9:
-                p[key] = value(-0.85,key)
-             if p[key]>=-0.1:
-                p[key] = value(-0.15,key)
+          # if k=='ovun5': 
+          #    if p[key]>=60.0:
+          #       p[key]= value(59.0,key)
+          #if k in ['bo2','bo4','bo6']:     ### scale bo
+             #if p[key]>=8.0:
+                #p[key] = value(7.95,key)
+             #if p[key]<=2.0:
+                #p[key] = value(2.05,key)
+          #if k in ['bo1','bo3','bo5']:     ### scale bo
+             #if p[key]<=-0.9:
+                #p[key] = value(-0.85,key)
+             #if p[key]>=-0.1:
+                #p[key] = value(-0.15,key)
           if k=='theta0':
-             if p[key]>=200.0:
-                p[key] = value(109.09,key)
+             if p[key]>=360.0:
+                p[key] = value(360.0,key)
           if k=='pen1':
              if p[key]>=200.0:
                 p[key] = value(100.0,key) 
