@@ -33,13 +33,17 @@ class MPNN(ReaxFF):
                cons=['val','vale',
                      # 'ovun1','ovun2','ovun3','ovun4',
                      # 'ovun5','ovun6','ovun7','ovun8',
-                     #'lp1','lp2','lp3',
+                     'lp2','lp3',#'lp1',
                      'cot1','cot2',
                      'coa1','coa2','coa3','coa4',
                      'pen1','pen2','pen3','pen4',
-                     'Depi','Depp','cutoff','acut'], # 'hbtol'
-               optmol=True,mefac=0.1,
+                     'Depi','Depp','cutoff','hbtol',
+                     #'val8','val9','val10','acut',
+                     ], # 
+               nn=True,
+               optmol=True,lambda_me=0.1,
                opt=None,optword='nocoul',
+               mpopt=None,bdopt=None,mfopt=None,
                VariablesToOpt=None,
                nanv={'boc1':-2.0},
                batch_size=200,sample='uniform',
@@ -47,32 +51,32 @@ class MPNN(ReaxFF):
                vdwcut=10.0,
                bore={'C-C':0.5,'others':0.45},
                bom={'others':1.2},
+               pim={'others':10.0},
                spv_bm=False,
                spv_be=False,
+               spv_pi=False,
+               spv_ang=False,
                weight={'others':1.0},
                ro_scale=0.1,
                clip_op=True,
                InitCheck=True,
                resetDeadNeuron=False,
-               nn=True,
                messages=1,
                TwoBodyOnly=False,
-               mpopt=None,
-               bo_layer=[4,1],
-               mf_layer=[9,2],
-               be_layer=[6,1],
-               be_univeral_nn=None,
-               bo_univeral_nn=None,
-               mf_univeral_nn=None,
-               vdw_layer=[4,1],  
+               be_univeral_nn=None,bo_layer=[4,1],
+               bo_univeral_nn=None,be_layer=[6,1],
+               mf_univeral_nn=None,mf_layer=[9,2],
+               vdw_univeral_nn=None,vdw_layer=None,#[6,1],
                vdwnn=False,
-               EnergyFunction=3,
+               EnergyFunction=1,
                MessageFunction=1,
                spec=[],
                sort=False,
                pkl=False,
                lambda_bd=100000.0,
+               lambda_pi=1.0,
                lambda_reg=0.0001,
+               lambda_ang=1.0,
                regularize=False,
                optMethod='ADAM',
                maxstep=60000,
@@ -94,32 +98,38 @@ class MPNN(ReaxFF):
       self.EnergyFunction   = EnergyFunction
       self.MessageFunction  = MessageFunction
       self.bom              = bom
+      self.pim              = pim
       self.spv_be           = spv_be
       self.spv_bm           = spv_bm
+      self.spv_pi           = spv_pi
+      self.spv_ang          = spv_ang
       self.TwoBodyOnly      = TwoBodyOnly
       self.regularize       = regularize 
       self.lambda_reg       = lambda_reg
+      self.lambda_pi        = lambda_pi
+      self.lambda_ang       = lambda_ang
       self.mf_layer         = mf_layer
       self.be_layer         = be_layer
-      self.vdw_layer        = vdw_layer
-
+      self.vdw_layer        = vdw_layer if vdwnn else None
       self.bo_univeral_nn   = bo_univeral_nn
       self.be_univeral_nn   = be_univeral_nn
       self.mf_univeral_nn   = mf_univeral_nn
-
-      self.vdwnn            = vdwnn
+      self.vdw_univeral_nn  = vdw_univeral_nn
       if mpopt is None:
          self.mpopt = [True for i in range(messages+3)]
       else:
          self.mpopt = mpopt
+      self.bdopt    = bdopt
+      self.mfopt    = mfopt
 
       ReaxFF.__init__(self,libfile=libfile,direcs=direcs,
                       dft=dft,atoms=atoms,cons=cons,opt=opt,optword=optword,
-                      VariablesToOpt=VariablesToOpt,optmol=optmol,mefac=mefac,
+                      VariablesToOpt=VariablesToOpt,optmol=optmol,lambda_me=lambda_me,
                       nanv=nanv,batch_size=batch_size,sample=sample,
                       hbshort=hbshort,hblong=hblong,vdwcut=vdwcut,
                       ro_scale=ro_scale,
-                      clip_op=clip_op,InitCheck=InitCheck,resetDeadNeuron=resetDeadNeuron,nn=nn,
+                      clip_op=clip_op,InitCheck=InitCheck,resetDeadNeuron=resetDeadNeuron,
+                      nn=nn,vdwnn=vdwnn,
                       bo_layer=bo_layer,spec=spec,sort=sort,pkl=pkl,weight=weight,
                       bore=bore,lambda_bd=lambda_bd,
                       optMethod=optMethod,maxstep=maxstep,
@@ -138,81 +148,125 @@ class MPNN(ReaxFF):
       ''' supervised learning term'''
       l_atol  = 0.0
       pen_w   = 0.0
-      # pen_b = 0.0
+      pen_b = 0.0
       wb_p    = ['fe','fsi','fpi','fpp']
-      w_n     = ['wi','wo']
+      if self.vdwnn:
+         wb_p.append('fv')
+      w_n     = ['wi','wo',]
       b_n     = ['bi','b','bo']
       layer   = {'fe':self.be_layer[1],'fsi':self.bo_layer[1],'fpi':self.bo_layer[1],
-                 'fpp':self.bo_layer[1] }
+                 'fpp':self.bo_layer[1]}
+      if self.vdwnn:
+         layer['fv'] = self.vdw_layer[1]
+
       wb_message = []
       for t in range(1,self.messages+1):
           wb_message.append('f'+str(t))
           layer['f'+str(t)] = self.mf_layer[1]
 
-      self.diffn,self.diffa,self.diffb,self.diffe,self.diffesi,self.diffe_= {},{},{},{},{},{}
+      self.diffn,self.diffa,self.diffb = {},{},{} 
+      self.diffpi,self.diffang = {},{}
+      self.diffe,self.diffesi,self.diffe_ = {},{},{}
+
       for bd in self.bonds: 
           [atomi,atomj] = bd.split('-') 
           if self.nbd[bd]>0:
-             if bd in self.bore:
-                bore_ = self.bore[bd]
+             bd_ = bd  if bd in self.bore else 'others'
+             if isinstance(self.bore[bd_],tuple):
+                re_ = self.bore[bd_][0]
+                bore_ = self.bore[bd_][1]
              else:
-                bore_ = self.bore['others']
+                bore_ = self.bore[bd_]
+                re_   = self.re[bd]
+ 
 
-             if bd in self.bom:
-                bm_ = self.bom[bd]
+             bd_ = bd  if bd in self.bom else 'others'
+             if isinstance(self.bom[bd_],tuple):
+                rm_  = self.bom[bd_][0]
+                bom_ = self.bom[bd_][1]
              else:
-                bm_ = self.bom['others']
+                rm_  = self.bom[bd_]
+                bom_ = bore_
+
              
              fao = tf.where(tf.greater(self.rbd[bd],self.rcuta[bd]),1.0,0.0)    ##### r> rcuta that bo = 0.0
-             self.diffn[bd]  = tf.reduce_sum(input_tensor=tf.nn.relu(self.bo0[bd]*fao - self.botol*fao))
+             self.diffn[bd]  = tf.reduce_sum(self.bo0[bd]*fao)
              l_atol = tf.add(self.diffn[bd]*self.lambda_bd,l_atol)
 
-             if self.spv_bm:
-                fao_= tf.where(tf.less_equal(self.rbd[bd],bm_*self.re[bd]),1.0,0.0) ##### r< 1.2*r_e that bo > botol
-                self.diffa[bd] = tf.reduce_sum(input_tensor=tf.nn.relu((self.botol-self.bo0[bd])*fao_))
+             if self.spv_bm: ############ smooth tail #########################
+                fao_= tf.where(tf.greater_equal(self.rbd[bd],rm_),1.0,0.0) ##### r > bm_ that bo < 10.0*botol
+                self.diffa[bd] = tf.reduce_sum(input_tensor=tf.nn.relu((self.bo0[bd] - bom_)*fao_))
                 l_atol = tf.add(self.diffa[bd]*self.lambda_bd,l_atol)
 
-             #  bop should be zero if r>rcut
+             # bop should be zero if r>rcut
              fbo = tf.where(tf.less_equal(self.rbd[bd],self.rc_bo[bd]),0.0,1.0) ##### r> rc_bo that bo < botol
-             self.diffb[bd]  = tf.reduce_sum(self.bop[bd]*fbo)
+             self.diffb[bd]  = tf.reduce_sum(self.bop[bd]*fbo) ##### Now the bop should be eterm1,eterm2,eterm3
              l_atol  = tf.add(self.diffb[bd]*self.lambda_bd,l_atol)
              
              if self.spv_be:
-                fe  = tf.where(tf.less_equal(self.rbd[bd],self.re[bd]),1.0,0.0) ##### r< r_e that bo > bore_
+                fe  = tf.where(tf.less_equal(self.rbd[bd],re_),1.0,0.0) ##### r< r_e that bo > bore_
                 self.diffe[bd]  = tf.reduce_sum(input_tensor=tf.nn.relu((bore_-self.bo0[bd])*fe))
                 l_atol  = tf.add(self.diffe[bd]*self.lambda_bd,l_atol)
             
                 self.diffe_[bd]  = tf.reduce_sum(input_tensor=tf.nn.relu((bore_-self.esi[bd])*fe))
-                l_atol  = tf.add(self.diffe_[bd]*self.lambda_bd,l_atol)          ## penalty iterm for bond energy
+                l_atol  = tf.add(self.diffe_[bd]*self.lambda_bd,l_atol)         ## penalty iterm for bond energy
 
-             if self.EnergyFunction == 1:
-                fe  = tf.where(tf.less(self.bo0[bd],self.botol),1.0,0.0) ##### bo <= 0.0 that e = 0.0
-                self.diffesi[bd]  = tf.reduce_sum(input_tensor=tf.nn.relu((self.esi[bd])*fe))
-                l_atol  = tf.add(self.diffesi[bd]*self.lambda_bd,l_atol)
+             # if self.EnergyFunction == 3:
+             #    fesi = tf.where(tf.less_equal(self.bo0[bd],self.botol),1.0,0.0) ##### bo <= 0.0 that e = 0.0
+             #    self.diffesi[bd]  = tf.reduce_sum(self.esi[bd]*fesi)
+             #    l_atol  = tf.add(self.diffesi[bd]*self.lambda_bd,l_atol)
              
              pen_rcut = tf.nn.relu(self.rc_bo[bd]-self.rcut[bd])
              l_atol= tf.add(pen_rcut*self.lambda_bd,l_atol)
-
+ 
              # penalize term for regularization of the neural networs
              if self.regularize:                                         # regularize
                 for k in wb_p:
                     for k_ in w_n:
                         key     = k + k_ + '_' + bd
                         pen_w  += tf.reduce_sum(tf.square(self.m[key]))
-
+                    for k_ in b_n:
+                        key     = k + k_ + '_' + bd
+                        pen_b  += tf.reduce_sum(tf.square(self.m[key]))
                     for l in range(layer[k]):                                               
                         pen_w += tf.reduce_sum(tf.square(self.m[k+'w_'+bd][l]))
+                        pen_b += tf.reduce_sum(tf.square(self.m[k+'b_'+bd][l]))
 
+      
+         # for sp in self.spec: 
+         #     pi_ = self.pim[sp] if sp in self.pim else self.pim['others']
+         #     if self.nsp[sp]>0:
+         #        self.diffpi[sp] = tf.reduce_sum(input_tensor=tf.nn.relu(self.Dpi[sp]-pi_))
+         #        l_atol  = tf.add(self.diffpi[sp]*self.lambda_pi,l_atol)
+      if self.optword.find('noang')<0:
+         for ang in self.angs: 
+             if self.nang[ang]>0:
+                if self.spv_pi:
+                   pi_ = self.pim[ang] if ang in self.pim else self.pim['others']
+                   self.diffpi[ang] = tf.reduce_sum(input_tensor=tf.nn.relu(self.SBO[ang]-pi_))
+                   l_atol  = tf.add(self.diffpi[ang]*self.lambda_pi,l_atol)
+                if self.spv_ang:
+                   self.diffang[ang] = tf.reduce_sum(self.thet2[ang]*self.fijk[ang])
+                   l_atol  = tf.add(self.diffang[ang]*self.lambda_ang,l_atol)
+
+         # self.diffpi = tf.reduce_sum(input_tensor=tf.nn.relu(self.DPI-2.0)) 
+         # l_atol  = tf.add(self.diffpi*self.lambda_pi,l_atol)
+ 
       if self.regularize:                                               # regularize
          for sp in self.spec:
              for k in wb_message:
                  for k_ in w_n:
                      key     = k + k_ + '_' + sp
                      pen_w  += tf.reduce_sum(tf.square(self.m[key]))
+                 for k_ in b_n:
+                     key     = k + k_ + '_' + sp
+                     pen_b  += tf.reduce_sum(tf.square(self.m[key]))
                  for l in range(layer[k]):                                               
                      pen_w += tf.reduce_sum(tf.square(self.m[k+'w_'+sp][l]))
+                     pen_b += tf.reduce_sum(tf.square(self.m[k+'b_'+sp][l]))
 
-      l_atol = tf.add(self.lambda_reg*pen_w,l_atol)
+         l_atol = tf.add(self.lambda_reg*pen_w,l_atol)
+         l_atol = tf.add(self.lambda_reg*pen_b,l_atol)
       return l_atol
 
   def get_loss(self):
@@ -249,10 +303,10 @@ class MPNN(ReaxFF):
           self.ME += tf.square(self.MolEnergy[mols])
 
       self.loss_atol = self.supervise()
-
       self.Loss     += self.loss_atol
+
       if self.optmol:
-         self.Loss  += self.ME*self.mefac
+         self.Loss  += self.ME*self.lambda_me
       self.accuracy  = self.accuracy/self.nmol
 
   def build_graph(self):
@@ -302,6 +356,31 @@ class MPNN(ReaxFF):
   #     self.EL[atom] = tf.math.divide(self.p['lp2_'+atom]*self.Delta_lp[atom],self.explp[atom],
   #                                    name='Elone_%s' %atom)
 
+  # def get_theta0(self,ang):
+  #     self.sbo[ang] = tf.gather_nd(self.DPI,self.dglist[ang])
+  #     #self.pbo[ang] = tf.gather_nd(self.PBO,self.dglist[ang])
+  #     #self.rnlp[ang]= tf.gather_nd(self.NLP,self.dglist[ang])
+  #     self.SBO[ang] = self.sbo[ang] #- tf.multiply(1.0-self.pbo[ang],self.D_ang[ang]+self.p['val8']*self.rnlp[ang])    
+      
+  #     ok         = tf.logical_and(tf.less_equal(self.SBO[ang],1.0),tf.greater(self.SBO[ang],0.0))
+  #     S1         = tf.where(ok,self.SBO[ang],tf.zeros_like(self.SBO[ang]))    #  0< sbo < 1                  
+  #     self.SBO01[ang] = tf.where(ok,tf.pow(S1,self.p['val9']),tf.zeros_like(S1)) 
+
+  #     ok    = tf.logical_and(tf.less(self.SBO[ang],2.0),tf.greater(self.SBO[ang],1.0))
+  #     S2    = tf.where(ok,self.SBO[ang],tf.zeros_like(self.SBO[ang]))                     
+  #     F2    = tf.where(ok,tf.ones_like(S2),tf.zeros_like(S2))                             #  1< sbo <2
+     
+  #     S2    = 2.0*F2-S2  
+  #     self.SBO12[ang] = tf.where(ok,2.0-tf.pow(S2,self.p['val9']),tf.zeros_like(self.SBO[ang]))     #  1< sbo <2
+  #                                                                                         #     sbo >2
+  #     SBO2  = tf.where(tf.greater_equal(self.SBO[ang],2.0),
+  #                      tf.ones_like(self.SBO[ang]),tf.zeros_like(self.SBO[ang]))
+
+  #     self.SBO3[ang]   = self.SBO01[ang]+self.SBO12[ang]+2.0*SBO2
+  #     theta0           = self.p['theta0_'+ang]*(1.0-tf.exp(-self.p['val10']*(2.0-self.SBO3[ang])))
+  #     self.theta0[ang] = theta0/57.29577951
+
+
   def get_total_energy(self):
       for mol in self.mols:
           # mols = mol.split('-')[0] 
@@ -353,6 +432,7 @@ class MPNN(ReaxFF):
 
   def get_tap(self,r,bd):
       if self.vdwnn:
+         # r_ = r/self.p['rvdw_'+bd]
          tp = self.f_nn('fv',bd,self.nvb[bd],[r],layer=self.vdw_layer[1])
       else:
          tp = 1.0+tf.math.divide(-35.0,tf.pow(self.vdwcut,4.0))*tf.pow(r,4.0)+ \
@@ -390,7 +470,7 @@ class MPNN(ReaxFF):
 
       self.bodiv1[bd] = tf.math.divide(self.rbd[bd],self.p['rosi_'+bd],name='bodiv1_'+bd)
       self.bopow1[bd] = tf.pow(self.bodiv1[bd],self.p['bo2_'+bd])
-      self.eterm1[bd] = (1.0+self.botol)*tf.exp(tf.multiply(self.p['bo1_'+bd],self.bopow1[bd]))*self.frc[bd] # consist with GULP
+      self.eterm1[bd] = tf.exp(tf.multiply(self.p['bo1_'+bd],self.bopow1[bd]))*self.frc[bd] 
 
       self.bodiv2[bd] = tf.math.divide(self.rbd[bd],self.p['ropi_'+bd],name='bodiv2_'+bd)
       self.bopow2[bd] = tf.pow(self.bodiv2[bd],self.p['bo4_'+bd])
@@ -422,7 +502,7 @@ class MPNN(ReaxFF):
       if self.MessageFunction==1:
          Fi    = self.f_nn(flabel,b[0],self.nbd[bd],[Dbi,Dbj,h],layer=self.mf_layer[1])
          Fj    = self.f_nn(flabel,b[1],self.nbd[bd],[Dbj,Dbi,h],layer=self.mf_layer[1])
-         F     = 2.0*Fi*Fj
+         F     = Fi*Fj
        
          bosi  = self.Hsi[t-1][bd]*F
          bopi  = self.Hpi[t-1][bd]*F
@@ -436,14 +516,6 @@ class MPNN(ReaxFF):
          bosi  = self.Hsi[t-1][bd]*Fsi
          bopi  = self.Hpi[t-1][bd]*Fpi
          bopp  = self.Hpp[t-1][bd]*Fpp
-      elif self.MessageFunction==3:
-         Fi    = self.f_nn(flabel,b[0],self.nbd[bd],[Dbi,Dbj,h],layer=self.mf_layer[1])
-         Fj    = self.f_nn(flabel,b[1],self.nbd[bd],[Dbj,Dbi,h],layer=self.mf_layer[1])
-         F     = Fi*Fj
-       
-         bosi  = self.Hsi[t-1][bd]*F
-         bopi  = self.Hpi[t-1][bd]*F
-         bopp  = self.Hpp[t-1][bd]*F
       else:
          raise NotImplementedError('-  Message function not supported yet!')
       bo       = bosi+bopi+bopp
@@ -551,24 +623,22 @@ class MPNN(ReaxFF):
       b       = bd.split('-')
       bdr     = b[1]+'-'+b[0]
 
-      if self.EnergyFunction==3:
+      if self.EnergyFunction==1:
          self.esi[bd] = self.f_nn('fe',bd, self.nbd[bd],[self.bosi[bd],self.bopi[bd],self.bopp[bd]],
                                   layer=self.be_layer[1])
          self.EBD[bd] = -self.p['Desi_'+bd]*self.esi[bd]*self.bo0[bd]
-      elif self.EnergyFunction==1:
+      elif self.EnergyFunction==2:
+         Fi = self.f_nn('fe',bd,self.nbd[bd],[Dbi,Dbj,self.bo0[bd]],layer=self.be_layer[1])
+         Fj = self.f_nn('fe',bd,self.nbd[bd],[Dbj,Dbi,self.bo0[bd]],layer=self.be_layer[1])
+         self.esi[bd] = Fi*Fj*self.bo0[bd]
+         self.EBD[bd] = -self.p['Desi_'+bd]*self.esi[bd]
+      elif self.EnergyFunction==3:
          self.esi[bd] = self.f_nn('fe',bd, self.nbd[bd],[self.bosi[bd],self.bopi[bd],self.bopp[bd]],
                                   layer=self.be_layer[1])
-         self.EBD[bd] = -self.p['Desi_'+bd]*self.esi[bd] # *self.bo0[bd]
-      elif self.EnergyFunction==2:
-         # Fi = self.f_nn('fe',bd, self.nbd[bd],[Dbi,Dbj,self.bosi[bd]],layer=self.be_layer[1])
-         # Fj = self.f_nn('fe',bdr,self.nbd[bd],[Dbj,Dbi,self.bosi[bd]],layer=self.be_layer[1])
-         # F  = Fi*Fj
-         # self.esi[bd]  = self.bosi[bd]*F       
-         self.sieng[bd]= tf.multiply(self.p['Desi_'+bd],self.bosi[bd])
-         self.pieng[bd]= tf.multiply(self.p['Depi_'+bd],self.bopi[bd])
-         self.ppeng[bd]= tf.multiply(self.p['Depp_'+bd],self.bopp[bd]) 
-
-         self.EBD[bd]  = - self.sieng[bd] - self.pieng[bd] - self.ppeng[bd] 
+         self.sieng[bd] = self.p['Desi_'+bd]*self.esi[bd]*self.bosi[bd]
+         self.pieng[bd] = self.p['Depi_'+bd]*self.esi[bd]*self.bopi[bd]
+         self.ppeng[bd] = self.p['Depp_'+bd]*self.esi[bd]*self.bopp[bd]
+         self.EBD[bd]   = - self.sieng[bd] - self.pieng[bd] - self.ppeng[bd] 
           
   def set_m(self):
       ''' set variable for neural networks '''
@@ -579,6 +649,12 @@ class MPNN(ReaxFF):
               bd = si + '-' + sj
               if bd not in bond:
                  bond.append(bd)
+
+      if self.mfopt is None:
+         self.mfopt = self.spec
+      if self.bdopt is None:
+          self.bdopt = self.bonds
+
       self.get_univeral_nn()
 
       reuse_m = True if self.bo_layer==self.bo_layer_ else False
@@ -600,7 +676,7 @@ class MPNN(ReaxFF):
                                     nin=3,nout=nout_,layer=self.mf_layer,
                                     nnopt=self.mpopt[t],bias=b)
           self.set_message_wb(pref='f'+str(t),reuse_m=reuse_m,nin=3,nout=nout_,
-                      vlist=bond,layer=self.mf_layer,nnopt=self.mpopt[t],bias=b) 
+                              layer=self.mf_layer,nnopt=self.mpopt[t],bias=b) 
 
       ############ set weight and bias for energy neural network ###################
       if self.EnergyFunction==self.EnergyFunction_ and self.be_layer==self.be_layer_:
@@ -616,6 +692,10 @@ class MPNN(ReaxFF):
 
       if self.vdwnn:
          reuse_m = True if self.vdw_layer==self.vdw_layer_ else False
+         if not self.vdw_univeral_nn is None:
+            self.set_univeral_wb(pref='fv',bd=self.vdw_univeral_nn[0],reuse_m=reuse_m,
+                                   nin=1,nout=1,layer=self.vdw_layer,
+                                   nnopt=self.mpopt[t+1],bias=-0.867)
          self.set_wb(pref='fv',reuse_m=reuse_m,nin=1,nout=1,layer=self.vdw_layer,
                      vlist=self.bonds,nnopt=self.mpopt[-1],bias=-0.867)
 
@@ -628,7 +708,7 @@ class MPNN(ReaxFF):
              self.m[pref+'wi_'+bd] = self.m[pref+'wi']
              self.m[pref+'bi_'+bd] = self.m[pref+'bi']
           elif pref+'wi_'+bd in self.m_ and reuse_m:                   # input layer
-             if nnopt:
+             if nnopt and (bd in self.bdopt):
                 self.m[pref+'wi_'+bd] = tf.Variable(self.m_[pref+'wi_'+bd],name=pref+'wi_'+bd)
                 self.m[pref+'bi_'+bd] = tf.Variable(self.m_[pref+'bi_'+bd],name=pref+'bi_'+bd)
              else:
@@ -646,7 +726,7 @@ class MPNN(ReaxFF):
                  self.m[pref+'w_'+bd] = self.m[pref+'w']
                  self.m[pref+'b_'+bd] = self.m[pref+'b']
           elif pref+'w_'+bd in self.m_ and reuse_m:     
-             if nnopt:                            
+             if nnopt and (bd in self.bdopt):                            
                 for i in range(layer[1]):   
                     self.m[pref+'w_'+bd].append(tf.Variable(self.m_[pref+'w_'+bd][i],name=pref+'wh'+str(i)+'_'+bd )) 
                     self.m[pref+'b_'+bd].append(tf.Variable(self.m_[pref+'b_'+bd][i],name=pref+'bh'+str(i)+'_'+bd )) 
@@ -665,7 +745,7 @@ class MPNN(ReaxFF):
              self.m[pref+'wo_'+bd] = self.m[pref+'wo']
              self.m[pref+'bo_'+bd] = self.m[pref+'bo']
           elif pref+'wo_'+bd in self.m_ and reuse_m:          # output layer
-              if nnopt:       
+              if nnopt and (bd in self.bdopt):       
                  self.m[pref+'wo_'+bd] = tf.Variable(self.m_[pref+'wo_'+bd],name=pref+'wo_'+bd)
                  self.m[pref+'bo_'+bd] = tf.Variable(self.m_[pref+'bo_'+bd],name=pref+'bo_'+bd)
               else:
@@ -676,15 +756,13 @@ class MPNN(ReaxFF):
               self.m[pref+'bo_'+bd] = tf.Variable(tf.random.normal([nout], stddev=0.0001)+bias,
                                                   name=pref+'bo_'+bd)
   
-  def set_message_wb(self,pref='f',reuse_m=True,nnopt=True,nin=8,nout=3,layer=[8,9],vlist=None,bias=0.0):
+  def set_message_wb(self,pref='f',reuse_m=True,nnopt=True,nin=8,nout=3,layer=[8,9],bias=0.0):
       ''' set matix varibles '''
       if self.m_ is None:
          self.m_ = {}
       for sp in self.spec:
           self.m[pref+'w_'+sp] = []                                    
           self.m[pref+'b_'+sp] = []
-          # self.m[pref+'w_'+sp+'-'+sp]  = []                                    
-          # self.m[pref+'b_'+sp+'-'+sp]  = []
           if pref+'_'+sp in self.univeral_nn:
              self.m[pref+'wi_'+sp] = self.m[pref+'wi']
              self.m[pref+'bi_'+sp] = self.m[pref+'bi']
@@ -693,7 +771,7 @@ class MPNN(ReaxFF):
              self.m[pref+'w_'+sp]  = self.m[pref+'w'] 
              self.m[pref+'b_'+sp]  = self.m[pref+'b'] 
           elif pref+'wi_'+sp in self.m_ and reuse_m:
-             if nnopt:
+             if nnopt and (sp in self.mfopt):
                 self.m[pref+'wi_'+sp] = tf.Variable(self.m_[pref+'wi_'+sp],name=pref+'wi_'+sp)
                 self.m[pref+'bi_'+sp] = tf.Variable(self.m_[pref+'bi_'+sp],name=pref+'bi_'+sp)
                 self.m[pref+'wo_'+sp] = tf.Variable(self.m_[pref+'wo_'+sp],name=pref+'wo_'+sp)
@@ -712,31 +790,13 @@ class MPNN(ReaxFF):
           else:
              self.m[pref+'wi_'+sp] = tf.Variable(tf.random.normal([nin,layer[0]],stddev=0.1),name=pref+'wi_'+sp)   
              self.m[pref+'bi_'+sp] = tf.Variable(tf.random.normal([layer[0]],stddev=0.1),name=pref+'bi_'+sp)  
-             self.m[pref+'wo_'+sp] = tf.Variable(tf.random.normal([nin,layer[0]],stddev=0.1),name=pref+'wo_'+sp)   
-             self.m[pref+'bo_'+sp] = tf.Variable(tf.random.normal([layer[0]],stddev=0.1),name=pref+'bo_'+sp)  
+             self.m[pref+'wo_'+sp] = tf.Variable(tf.random.normal([layer[0],nout],stddev=0.1),name=pref+'wo_'+sp)   
+             self.m[pref+'bo_'+sp] = tf.Variable(tf.random.normal([nout],stddev=0.1),name=pref+'bo_'+sp)  
              for i in range(layer[1]):   
                  self.m[pref+'w_'+sp].append(tf.Variable(tf.random.normal([layer[0],layer[0]], 
-                                                         stddev=0.1),name=pref+'wh'+str(i)+'_'+bd)) 
+                                                         stddev=0.1),name=pref+'wh'+str(i)+'_'+sp)) 
                  self.m[pref+'b_'+sp].append(tf.Variable(tf.random.normal([layer[0]], 
-                                                         stddev=0.1),name=pref+'bh'+str(i)+'_'+bd )) 
-
-          # self.m[pref+'wi_'+sp+'-'+sp] = self.m[pref+'wi_'+sp]
-          # self.m[pref+'bi_'+sp+'-'+sp] = self.m[pref+'bi_'+sp]
-          # self.m[pref+'wo_'+sp+'-'+sp] = self.m[pref+'wo_'+sp]
-          # self.m[pref+'bo_'+sp+'-'+sp] = self.m[pref+'bo_'+sp]
-          # self.m[pref+'w_'+sp+'-'+sp]  = self.m[pref+'w_'+sp]
-          # self.m[pref+'b_'+sp+'-'+sp]  = self.m[pref+'b_'+sp]
-
-      # for bd in vlist:
-      #     b = bd.split('-')
-      #     sp= b[0]
-      #     if b[0]!=b[1]:                   # input layer
-      #        self.m[pref+'wi_'+bd] = self.m[pref+'wi_'+sp]
-      #        self.m[pref+'bi_'+bd] = self.m[pref+'bi_'+sp]
-      #        self.m[pref+'wo_'+bd] = self.m[pref+'wo_'+sp]
-      #        self.m[pref+'bo_'+bd] = self.m[pref+'bo_'+sp]
-      #        self.m[pref+'w_'+bd]  = self.m[pref+'w_'+sp]
-      #        self.m[pref+'b_'+bd]  = self.m[pref+'b_'+sp]
+                                                         stddev=0.1),name=pref+'bh'+str(i)+'_'+sp)) 
 
 
   def set_univeral_wb(self,pref='f',bd='C-C',reuse_m=True,nnopt=True,nin=8,nout=3,
@@ -812,7 +872,18 @@ class MPNN(ReaxFF):
              bdr = b[1] + '-' + b[0]
              self.univeral_nn.append('fe_'+bd)
              self.univeral_nn.append('fe_'+bdr)
-             
+
+      if not self.vdw_univeral_nn is None:
+         if self.vdw_univeral_nn=='all':
+            univeral_bonds = self.bonds
+         else:
+            univeral_bonds = self.vdw_univeral_nn
+         for bd in univeral_bonds:
+             b = bd.split('-')
+             bdr = b[1] + '-' + b[0]
+             self.univeral_nn.append('fv_'+bd)
+             self.univeral_nn.append('fv_'+bdr)
+
       if not self.mf_univeral_nn is None:
          if self.mf_univeral_nn=='all':
             univeral_bonds = self.spec
@@ -862,10 +933,10 @@ class MPNN(ReaxFF):
               'mf_layer':self.mf_layer,
               'be_layer':self.be_layer,
               'vdw_layer':self.vdw_layer,
-              'MolEnergy':self.MolEnergy_,
               'rcut':self.rcut,
               'rcutBond':self.rcuta,
-              'rEquilibrium':self.re}
+              'rEquilibrium':self.re,
+              'MolEnergy':self.MolEnergy_}
          js.dump(j,fj,sort_keys=True,indent=2)
          fj.close()
       else:
