@@ -75,8 +75,8 @@ def e(traj='md.traj',efunc=3,fm=1,batch_size=50,nn=True):
               InitCheck=False,
               nn=nn,
               bo_layer=[4,1],
-              bf_layer=[6,2],
-              be_layer=[3,1],
+              mf_layer=[9,3],
+              be_layer=[6,1],
               EnergyFunction=efunc,
               MessageFunction=fm) 
     molecules = rn.initialize()
@@ -101,8 +101,8 @@ def e(traj='md.traj',efunc=3,fm=1,batch_size=50,nn=True):
     da_  = rn.get_value(rn.diffa)
     de_  = rn.get_value(rn.diffe)
 
-    for bd in db_:
-        print(bd,db_[bd],da_[bd],de_[bd])
+    # for bd in db_:
+    #     print(bd,db_[bd],da_[bd],de_[bd])
 
     e_irff = {'Energy':e,'Ebond':eb,'Eunger':eu,'Eover':eo,'Eang':ea,'Epen':ep,
               'Elone':el,
@@ -167,7 +167,9 @@ def ep(traj='opt.traj',efun=3,batch_size=50,nn=True):
                 clip_op=False,
                 InitCheck=False,
                 nn=nn,
-                bo_layer=[9,2],
+                bo_layer=[4,1],
+                mf_layer=[9,3],
+                be_layer=[6,1],
                 EnergyFunction=efun,
                 pkl=False) 
     molecules = rn.initialize()
@@ -213,7 +215,7 @@ def ep(traj='opt.traj',efun=3,batch_size=50,nn=True):
         plt.close() 
 
 
-def db(traj='opt.traj',batch_size=50,ef=3,nn=True,frame=40):
+def db(traj='md.traj',batch_size=50,ef=3,fm=1,nn=True,frame=15):
     ffield = 'ffield.json' if nn else 'ffield'
     images = Trajectory(traj)
     atoms  = images[frame]
@@ -230,17 +232,19 @@ def db(traj='opt.traj',batch_size=50,ef=3,nn=True,frame=40):
     ebond = ir.ebond # .numpy()
 
     rn = MPNN(libfile=ffield,
-                direcs={'tmp':traj},
-                dft='siesta',
-                opt=[],optword='nocoul',
-                batch_size=batch_size,
-                atomic=True,
-                clip_op=False,
-                InitCheck=False,
-                nn=nn,
-                bo_layer=[9,2],
-                EnergyFunction=ef,
-                pkl=False) 
+              direcs={'tmp':traj},
+              dft='siesta',
+              opt=[],optword='nocoul',
+              batch_size=batch_size,
+              clip_op=False,
+              InitCheck=False,
+              nn=nn,
+              bo_layer=[4,1],
+              mf_layer=[9,3],
+              be_layer=[6,1],
+              EnergyFunction=ef,
+              MessageFunction=fm) 
+
     molecules = rn.initialize()
     rn.session(learning_rate=1.0e-10,method='AdamOptimizer')
 
@@ -249,9 +253,9 @@ def db(traj='opt.traj',batch_size=50,ef=3,nn=True,frame=40):
     blab = rn.lk.bdlab
 
     bosi   = ir.bosi.numpy()
-    bopsi = ir.bop_si.numpy()
-    boppi = ir.bop_pi.numpy()
-    boppp = ir.bop_pp.numpy()
+    bopsi  = ir.bop_si.numpy()
+    boppi  = ir.bop_pi.numpy()
+    boppp  = ir.bop_pp.numpy()
 
     bopow3 = ir.bopow3.numpy()
     eterm3 = ir.eterm3.numpy()
@@ -305,20 +309,6 @@ def db(traj='opt.traj',batch_size=50,ef=3,nn=True,frame=40):
     eb   = rn.get_value(rn.ebond[mol])
     print('\n-  bond energy:',ir.Ebond.numpy(),eb[frame],end='\n')
 
-    for i in range(ir.natom):
-        for j in range(ir.natom):
-            if abs(ebd_[i][j] - ebond[i][j])>0.00001:
-               print('-  %s %2d %2d:' %(bd,i,j),
-                      'rbd %10.7f' %(r[i][j]),
-                      'bop %10.7f' %(bop[i][j]),
-                      'Di %10.7f %10.7f' %(D_[i][frame],D[i]),
-                      'Dj %10.7f %10.7f' %(D_[j][frame],D[j]),
-                      'bo %10.7f' %(bo[i][j]),
-                      'ebond %10.7f %10.7f' %(ebd_[i][j],ebond[i][j])  )
-
-    # print(rcbo)
-    print('\n-  bond energy:',ir.Ebond.numpy(),eb[frame],end='\n')
-
   
 def dl(traj='siesta.traj',batch_size=1,nn=False,frame=0):
     ffield = 'ffield.json' if nn else 'ffield'
